local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local LSM = LibStub("LibSharedMedia-3.0")

local Boss = RW.bossPrototype
local warningProt = {}
local mt = {__index = warningProt}

local Party = RW:GetPlugin("Party")

Boss.BITS = Boss.BITS or {}
Boss.BITS.WARNING = {
	NORMAL	= 0x00000001,
	CAST 	= 0x00000002,
	RUNAWAY = 0x00000004,
}

-- If text is a number this warning is considered not a custom one
-- If text is not a number then it is custom and it can be possibly sent as a table,
-- then the first value in the table will be the actual text, and the second oen will be the option text as its show in the GUI
-- 3rd value in the table will be the tooltip text
local spell
function Boss:_Warning(wType, text, icon, optionName, default, color, sound)
	if type(text) == "number" then 
		spell = text
		return self:_Warning(wType, GetSpellInfo(text), icon or text, optionName or text, default, color, sound) 
	end
	if type(icon) == "number" then 
		spell = icon
		return self:_Warning(wType, text, select(3, GetSpellInfo(icon)), optionName or icon, default, color, sound) 
	end
	
	optionName = type(optionName) == "number" and RW.SpellName[optionName] or optionName
	
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	self:AddToCategory(nil, optionName)
	
	local defaults = self:AddDefaults(optionName, "WARNING", wType ~= "CUSTOM" and wType or text[2], default)
	defaults.flash = true
	defaults.color = color
	if wType == "RUNAWAY" and not sound 
		then defaults.sound  = "RW: RunAwayLittleGirl"
	else
		defaults.sound = "RW: Tata"
	end
	
	local base = self:GetBaseAlertObject(optionName)
	if not base then
		base = {
			icon = icon,			
			id = optionName,
			spell = spell,
			mod = self,
			scheduled = {},
		}
		tinsert(self.BaseAlertObjects, base)
	end
	
	local obj = setmetatable({
		mainType = "WARNING",
		type = wType,		
		text = type(text) == "table" and text[1] or text,
		optText = type(text) == "table" and text[2] or nil,
		base = base,
	}, mt)
	
	self.AlertObjects.WARNING = self.AlertObjects.WARNING or {}
	tinsert(self.AlertObjects.WARNING, obj)
	
	spell = nil
	
	return obj
end

function Boss:Warning(...)
	return self:_Warning("NORMAL", ...)
end

function Boss:RunawayWarning(...)
	return self:_Warning("RUNAWAY", ...)
end

function Boss:CastWarning(...)
	return self:_Warning("CAST", ...)
end

function Boss:CustomWarning(text, optText, ...)
	--@debug
	assert(type(text) == "string", self:GetName()..": CustomTimer, variable \"text\" is not of type text")
	assert(type(optText) == "string", self:GetName()..": CustomTimer, variable \"optText\" is not of type text")
	--@end-debug
	return self:_Warning("CUSTOM", {text, optText}, ...)
end

function warningProt:Show(...)
	if not self.base.mod.DB[self.base.id].enabled then return end
	if not self.base.mod.DB[self.base.id].WARNING[self.optText or self.type].enabled then return end
		
	local text
	if self.optText then	
		text = RW:Format(self.text, ...)
	else
		text = RW:Format(L["FORMAT_WARNING_"..self.type], self.text or ..., ...)
	end
	
	text = RW:ColorNames(text)
	
	local db = self.base.mod.DB
	
	self.base.mod:SendMessage("RW_Warning", text, self.base.icon, db[self.base.id].WARNING[self.optText or self.type].color)
	-- Play sound
	if db[self.base.id].WARNING[self.optText or self.type].sound then 
		PlaySoundFile(LSM:Fetch("sound", db[self.base.id].WARNING[self.optText or self.type].sound)) 
	end
	-- Flash the screen
	if db[self.base.id].WARNING[self.optText or self.type].flash then
		self.base.mod:SendMessage("RW_Flash", db[self.base.id].WARNING[self.optText or self.type].color)
	end
end

local function showMessage(message, ...)
	message:Show(...)
end

function warningProt:Schedule(stime, info, ...)
	self.base.mod:Schedule("WARNING"..self.base.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.base.scheduled, "WARNING"..self.base.id..(info or ""))
end

function warningProt:ScheduleRepeating(stime, now, info, ...)
	self.base.mod:ScheduleRepeating("WARNING"..self.base.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.base.scheduled, "WARNING"..self.base.id..(info or ""))
end

function warningProt:Unschedule(extraID)
	self.base.mod:Unschedule("WARNING"..self.base.id..(extraID or ""))
	RW:TRemove(self.base.scheduled, "WARNING"..self.base.id..(extraID or ""))
end

-- Options
function warningProt:GetOption()
	local Options = RW:GetPlugin("Options")
	local boss = self.base.mod
	local AceGUI = LibStub("AceGUI-3.0")
	
	local subGroup = AceGUI:Create("OptionFrame")
	subGroup:SetFullWidth(true)
	subGroup:SetLayout("flow")
	
	
	local check = AceGUI:Create("CheckBox")
	if self.type == "CUSTOM" then
		subGroup:SetTitle(self.optText)				
	else
		subGroup:SetTitle(L["WARNING_"..self.type])
		check:SetCallback("OnEnter", Options.OptionHelpers.OnOptionEnter)
		check:SetCallback("OnLeave",  Options.OptionHelpers.OnOptionLeave)
	end			
	check:SetUserData("obj", self)
	check:SetCallback("OnValueChanged", Options.OptionHelpers.OnValueChanged)
	check:SetValue(boss.DB[self.base.id].WARNING[self.optText or self.type].enabled)
	check:SetLabel("Enable")
				
	local color = AceGUI:Create("ColorPicker")
	local colorvalue = boss.DB[self.base.id].WARNING[self.optText or self.type].color or nil
	if colorvalue then 
		color:SetColor(unpack(colorvalue))
	end
	color:SetLabel("Color")
	color:SetUserData("obj", self)
	color:SetCallback("OnValueConfirmed", Options.OptionHelpers.OnColorSelected)
	
	local drop = AceGUI:Create("LSM30_Sound")
	drop:SetRelativeWidth(0.39)
	drop:SetList()
	drop:SetLabel("Sound")
	drop:SetUserData("obj", self)
	drop:SetUserData("key", "sound")
	drop:SetCallback("OnValueChanged", Options.OptionHelpers.OnKeyValueChanged)
	drop:SetValue(boss.DB[self.base.id].WARNING[self.optText or self.type].sound)

	local flash = AceGUI:Create("CheckBox")
	flash:SetRelativeWidth(0.35)
	flash:SetLabel("Screen Flash")
	flash:SetUserData("obj", self)
	flash:SetUserData("key", "sound")
	flash:SetCallback("OnValueChanged", Options.OptionHelpers.OnKeyValueChanged)
	flash:SetValue(boss.DB[self.base.id].WARNING[self.optText or self.type].flash)
	
	subGroup:AddChild(check)
	subGroup:AddChild(color)
	subGroup:AddChild(flash)
	subGroup:AddChild(drop)
	
	return subGroup
end 